/* OS/2 compatibility functions.
   Copyright (C) 1995-1998, 2000, 2001 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU Library General Public License as published
   by the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

#define __OS2_AWARE__
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <string.h>

/* A version of getenv() that works from DLLs */
extern unsigned long DosScanEnv (const unsigned char *pszName, unsigned char **ppszValue);

char *dll_getenv (const char *name)
{
  unsigned char *value;
  if (DosScanEnv (name, &value))
    return NULL;
  else
    return value;
}

char *_os2_libdir = NULL;
char *_os2_localealiaspath = NULL;
char *_os2_localedir = NULL;

static __attribute__((constructor)) void os2_initialize ()
{
  char *root = getenv ("UNIXROOT");
  char *gnulocaledir = getenv ("GNULOCALEDIR");

  _os2_libdir = gnulocaledir;
  if (!_os2_libdir)
    {
      if (root)
        {
#define LIBDIR_LEN	strlen (LIBDIR)
          int sl = strlen (root);
          _os2_libdir = (char *)malloc (sl + LIBDIR_LEN + 1);
          memcpy (_os2_libdir, root, sl);
          memcpy (_os2_libdir + sl, LIBDIR, LIBDIR_LEN + 1);
        }
      else
        _os2_libdir = LIBDIR;
    }

  _os2_localealiaspath = gnulocaledir;
  if (!_os2_localealiaspath)
    {
      if (root)
        {
#define LOCALE_ALIAS_PATH_LEN strlen (LOCALE_ALIAS_PATH)
          int sl = strlen (root);
          _os2_localealiaspath = (char *)malloc (sl + LOCALE_ALIAS_PATH_LEN + 1);
          memcpy (_os2_localealiaspath, root, sl);
          memcpy (_os2_localealiaspath + sl, LOCALE_ALIAS_PATH, LOCALE_ALIAS_PATH_LEN + 1);
        }
     else
        _os2_localealiaspath = LOCALE_ALIAS_PATH;
    }

  _os2_localedir = gnulocaledir;
  if (!_os2_localedir)
    {
      if (root)
        {
#define LOCALEDIR_LEN strlen (LOCALEDIR)
          int sl = strlen (root);
          _os2_localedir = (char *)malloc (sl + LOCALEDIR_LEN + 1);
          memcpy (_os2_localedir, root, sl);
          memcpy (_os2_localedir + sl, LOCALEDIR, LOCALEDIR_LEN + 1);
        }
      else
        _os2_localedir = LOCALEDIR;
    }

  extern const char *_nl_default_dirname__;
  _nl_default_dirname__ = _os2_localedir;
}

#ifdef HAVE_ICONV
#include "support/os2/iconv.c"
#endif

#ifdef HAVE_LANGINFO_CODESET
#include <langinfo.h>
#define INCL_DOS
#include <os2.h>
#include <stdio.h>
const char *nl_langinfo (int category)
{
  switch (category)
    {
      case CODESET:
        {
          ULONG cp [3], cplen;
          if (DosQueryCp (sizeof (cp), cp, &cplen))
            break;

          static char codeset [20];
          sprintf (codeset, "IBM-%ld", cp [0]);
          return codeset;
        }
    }
  return NULL;
}
#endif
